import 'dart:convert';

import 'package:axlehire_flutter_model/axlehire_model.dart';
import 'package:axlehire_flutter_model/src/address.dart';
import 'package:axlehire_flutter_model/src/client.dart';
import 'package:axlehire_flutter_model/src/delivery_reason.dart';
import 'package:axlehire_flutter_model/src/mixin/metadata.dart';
import 'package:axlehire_flutter_model/src/shipment.dart';
import 'package:axlehire_flutter_model/src/shipment_image.dart';
import 'package:axlehire_flutter_model/src/shipment_signature.dart';
import 'package:axlehire_flutter_model/src/stop.dart';
import 'package:json_annotation/json_annotation.dart';
part 'decor_stop.g.dart';

@JsonSerializable()

class DecorStop with MetaDataMixin {
  DecorStop(
    this.stop,
    this.shipment,
    this.label,
    this.client,
    { this.deliveryStatus, this.images, this.signatures, this.address, this.geocodeAddress }
  ) {
    this.address = this.shipment == null || this.stop == null ? null : (this.stop.type == 'DROP_OFF' || this.stop.type == 'REATTEMPT' ? this.shipment.dropoffAddress : this.shipment.pickupAddress);
  }

  final Stop stop;
  final Shipment shipment;
  final String label;
  final Client client;

  @JsonKey(name: 'delivery_status')
  Map<String, String> deliveryStatus;

  List<ShipmentImage> images;
  List<ShipmentSignature> signatures;

  bool scanned;
  Address address;
  GeocodeAddress geocodeAddress;

  DecorStop clone({Shipment shipment, Stop stop, String label, Client client, Map<String, String> deliveryStatus, Map<String, dynamic> meatadata, Address address, GeocodeAddress geocodeAddress, List<ShipmentImage> images, List<ShipmentSignature> signatures}) =>
    DecorStop(
      stop ?? this.stop,
      shipment ?? this.shipment,
      label ?? this.label,
      client ?? this.client,
      deliveryStatus: deliveryStatus ?? this.deliveryStatus,
      images: images ?? this.images,
      signatures: signatures ?? this.signatures,
      address: address ?? this.address,
      geocodeAddress: geocodeAddress ?? this.geocodeAddress
    )..metadata = metadata ?? this.metadata;

  /// convenient functions
  DecorStop withShipment(Shipment shipment) => clone(shipment: shipment);
  DecorStop withStop(Stop stop) => clone(stop: stop);
  DecorStop withLabel(String label) => clone(label: label);
  DecorStop withClient(Client client) => clone(client: client);
  DecorStop withMetadata(Map<String, dynamic> metadata) => clone(meatadata: metadata);
  DecorStop withDeliveryStatus(Map<String, String> statusUpdates) {
    Map<String, String> statuses = this.deliveryStatus == null ? {} : Map.from(this.deliveryStatus);
    statuses.addAll(statusUpdates);
    return clone(deliveryStatus: statuses);
  }
  DecorStop withImages(List<ShipmentImage> images) => clone(images: images);
  DecorStop addImages(List<ShipmentImage> images) {
    List<ShipmentImage> allImages = this.images ?? [];
    allImages.addAll(images);
    return clone(images: allImages);
  }

  DecorStop withSignatures(List<ShipmentSignature> signatures) => clone(signatures: signatures);
  DecorStop addSignatures(List<ShipmentSignature> signatures) {
    List<ShipmentSignature> allSignatures = this.signatures ?? [];
    allSignatures.addAll(signatures);
    return clone(signatures: allSignatures);
  }

  Map<String, List<DeliveryReason>> get deliveryReasons {
    if (metadata == null || metadata.isEmpty || !metadata.containsKey('DELIVERY_REASON')) return {};
    Map<String, dynamic> deliveryReasons = metadata['DELIVERY_REASON'] as Map<String, dynamic>;
    return deliveryReasons.map<String, List<DeliveryReason>>(
      (k,v) => MapEntry(k, (v as List<dynamic>).map((r) => DeliveryReason.fromJson(r as Map<String, dynamic>)).toList())
    );
  }

  List<DeliveryReason> getReasonsByStatus(String status) {
    if (metadata == null || metadata.isEmpty || !metadata.containsKey('DELIVERY_REASON')) return [];
    Map<String, dynamic> deliveryReasons = metadata['DELIVERY_REASON'] as Map<String, dynamic>;
    if (!deliveryReasons.containsKey(status)) return [];
    List<dynamic> reasons = deliveryReasons[status];
    return reasons.map((r) => DeliveryReason.fromJson(r as Map<String, dynamic>)).toList();
  }

  DeliveryReason get selectedReason {
    if (deliveryStatus == null || deliveryStatus.isEmpty || !deliveryStatus.containsKey('reason')) return null;
    return DeliveryReason.fromJson(json.decode(deliveryStatus['reason']));
  }

  DecorStop withReason(DeliveryReason reason) {
    return withDeliveryStatus({'reason': json.encode(reason)});
  }

  // get metadata
  bool get shouldPreview => getMetaBool('preview', defaultValue: false);
  bool get didPreview => deliveryStatus != null && deliveryStatus.containsKey('preview');

  bool get scanRequired => getMetaBool("scan_required", defaultValue: false);

  bool get showParcels => shipment != null && shipment.parcels != null && shipment.parcels.isNotEmpty && getMetaBool('show_parcel', defaultValue: false);
  bool get shouldHandleParcels => shipment != null && shipment.parcels != null && shipment.parcels.isNotEmpty && getMeta('handling_level') == 'parcel';

  String get receiveStatus => deliveryStatus == null ? null : deliveryStatus['receive-status'];
  bool get isReceivedOk => this.receiveStatus == 'ok';
  bool get isReceivedDamaged => this.receiveStatus == 'damaged';
  bool get isMissing => this.receiveStatus == 'missing';
  bool get didReceive => deliveryStatus != null && deliveryStatus.containsKey('received');

  String receiveParcelStatus(String id) => deliveryStatus == null ? null : deliveryStatus['receive[$id]-status'];
  bool isParcelReceivedOk(String id) => receiveParcelStatus(id) == 'ok';
  bool isParcelReceivedDamaged(String id) => receiveParcelStatus(id) == 'damaged';
  bool isParcelMissing(String id) => receiveParcelStatus(id) == 'missing';
  bool didReceiveParcel(String id) => deliveryStatus != null && deliveryStatus.containsKey('received[$id]');

  factory DecorStop.fromJson(Map<String, dynamic> json) => _$DecorStopFromJson(json);
  Map<String, dynamic> toJson() => _$DecorStopToJson(this);
}